#---------------------------------------------------------------------
# authentication & authorization
#---------------------------------------------------------------------

package AbsenceAuthorization;

BEGIN {
	# an instance has been specified
	if (exists($ENV{INSTANCE_NAME})) {
        $cookie_name = "absence-auth-$ENV{INSTANCE_NAME}";
    } else {
        $cookie_name = 'absence-auth';
    }
}

use AbsenceConfig;
use AbsenceLog;

my $AUTH		= AbsenceConfig::fetch('authentication');
#my $AUTH_TYPE	= AbsenceConfig::fetch('auth_type');
#my $SECRET		= AbsenceConfig::fetch('secret');
#my $TOP_PAGE	= AbsenceConfig::fetch('top_page');

my $DEBUG		= 0;
my $VERSION		= '1.0';

sub dbg
{
	my $msg = shift;

	$DEBUG && abslog("DEBUG: $msg");
}

sub allowWriteForPerson
{
	my ($user, $pid) = @_;

	$AUTH || return 1;

	my $acl = AbsenceDB::getAcl($user);
	my $self = AbsenceDB::getPidForUser($user);

	#---------------------------------------------------------------
	# check if $user is working on self and "w:self" in person list
	#---------------------------------------------------------------
	my $p = ','.join(',', @{$acl->{p}}).',';
	if (($pid == $self) && ($p =~ /,w:self,/)) {
		return 1;
	} elsif ($p =~ /,w:$pid,/) {
		return 1;
	}
	
	my $g = ','.join(',', @{$acl->{g}}).',';

	#---------------------------------------------------------------
	# check if $pid is in a group of which $self is a member and
	# "w:self" or "a:self" in group list
	#---------------------------------------------------------------
	my $targ_grps = AbsenceDB::getPerson($pid, 'group');
	if ($self && ($g =~ /,(a|w):self,/)) {
		my $targ_grps = AbsenceDB::getPerson($pid, 'group');
		my $gref = AbsenceDB::getPerson($self, 'group');
		foreach my $grp (@{$gref}) {
			if (inList($grp, $targ_grps)) { return 1; }
		}
	}

	#---------------------------------------------------------------
	# is "w:<gid>" or "a:<gid>" in group list?
	#---------------------------------------------------------------
	foreach my $grp (@{$targ_grps}) {
		if ($g =~ /,(w|a):$grp,/) { return 1; }
	}

	if ($g =~ /,(a|w):all,/) { return 1; }

	return 0;
}

sub allowReadForGroup
{
	my ($user, $gid) = @_;

	$AUTH || return 1;

	my $acl = AbsenceDB::getAcl($user);
	my $self = AbsenceDB::getPidForUser($user);

	my $g = ','.join(',', @{$acl->{g}}).',';

	#---------------------------------------------------------------
	# check if $gid is a group of which $self is a member
	#---------------------------------------------------------------
	if ($self) {
		my $gref = AbsenceDB::getPerson($self, 'group');
		foreach my $grp (@{$gref}) {
			if ($gid == $grp) { return 1; }
		}
	}

	if ($g =~ /,(a|w|r):($gid|all),/) {
		return 1;
	}

	return 0;
}

sub allowWriteForGroup
{
	my ($user, $gid) = @_;

	$AUTH || return 1;

	my $acl = AbsenceDB::getAcl($user);
	my $self = AbsenceDB::getPidForUser($user);

	my $g = ','.join(',', @{$acl->{g}}).',';

	#---------------------------------------------------------------
	# check if $gid is a group of which $self is a member
	#---------------------------------------------------------------
	if ($self && ($g =~ /,(a|w):self,/)) {
		my $gref = AbsenceDB::getPerson($self, 'group');
		foreach my $grp (@{$gref}) {
			if ($gid == $grp) { return 1; }
		}
	}

	if ($g =~ /,(a|w):($gid|all),/) {
		return 1;
	}

	return 0;
}

sub allowAdminForGroup
{
	my ($user, $gid) = @_;

	$AUTH || return 1;

	my $acl = AbsenceDB::getAcl($user);
	my $self = AbsenceDB::getPidForUser($user);

	my $g = ','.join(',', @{$acl->{g}}).',';

	#---------------------------------------------------------------
	# check if $gid is a group of which $self is a member
	#---------------------------------------------------------------
	if ($self && ($g =~ /,a:self,/)) {
		my $gref = AbsenceDB::getPerson($self, 'group');
		foreach my $grp (@{$gref}) {
			if ($gid == $grp) { return 1; }
		}
	}

	if ($g =~ /,a:($gid|all),/) {
		return 1;
	}

	return 0;
}

#-----------------------------------------------------------------
# what groups is $user allowed to see?
#-----------------------------------------------------------------
sub getReadGroups
{
	my $user = shift;

	$AUTH || return (AbsenceDB::getGroups());

	my $acl = AbsenceDB::getAcl($user);
	my $self = AbsenceDB::getPidForUser($user);

	my %unique_gids;

	#---------------------------------------------------------
	# add all groups in which $self is a member
	# (implicit "r:self")
	#---------------------------------------------------------
	if ($self) {
		foreach my $grp (@{AbsenceDB::getPerson($self, 'group')}) {
			$unique_gids{$grp} = 1;
		}
	}

	foreach my $rule (@{$acl->{g}}) {
		if ($rule =~ /^(a|w|r):all$/) { return (AbsenceDB::getGroups()); }
		if ($rule =~ /^(?:a|w|r):(.*)$/) {
			my $rhs = $1;
			if ($rhs eq 'self') {
				if ($self) {
					my $gref = AbsenceDB::getPerson($self, 'group');
					foreach my $grp (@{$gref}) {
						$unique_gids{$grp} = 1;
					}
				}
			} else {
				$unique_gids{$rhs} = 1;
			}
		}
	}

	return (keys(%unique_gids));
}

#-----------------------------------------------------------------
# what people is $user allowed to manage?
#-----------------------------------------------------------------
sub getAdminPeople
{
	my ($user, $op) = @_;

	$AUTH || return (AbsenceDB::getPeople(0));
	
	#my $acl = AbsenceDB::getAcl($user);
	#my $self = AbsenceDB::getPidForUser($user);

	my @admgrps = getAdminGroups($user);

	#-------------------------------------------------------------
	# first, make a list of all person-IDs in all gropus for
	# which $user has admin privs.  Additionally, if the
	# operation is "delete", people are not included in the
	# list if they belong to groups for which $user does not
	# have any admin privs.
	#-------------------------------------------------------------
	my %unique_pids;
	foreach my $grp (@admgrps) {
		foreach my $pid (AbsenceDB::getPeople($grp)) {
			if (adminAll($user) || ($op ne 'delete')) {
				$unique_pids{$pid} = 1;
			} elsif (adminForAllGroups($pid, \@admgrps)) {
				# ROBBO
				$unique_pids{$pid} = 1;
			}
		}
	}

	#-------------------------------------------------------------
	# group-admins have no admin rights for super-admins.  Go
	# through list and remove all people with "admin:all" privs.
	#-------------------------------------------------------------
	if (!adminAll($user)) {
		foreach my $pid (keys(%unique_pids)) {
			my $uid = AbsenceDB::getPerson($pid, 'user_id') || next;;
			if (adminAll($uid)) {
				delete $unique_pids{$pid};
			}
		}
	}

	return (keys(%unique_pids));
}

#-----------------------------------------------------------------
# getAdminUsers() returns a list of users whom $user is allowed
# to administer.  The list depends on what the admin operation
# is.  If the operation ($op) is "delete", a list consists of
# all the users that only have access to groups for which $user
# has admin privs.  Thus, a group administrator can only delete
# a user if the user only has read/write/admin privs for groups
# that the group administrator has admin privs for...
#-----------------------------------------------------------------
sub getAdminUsers
{
	my ($user, $op) = @_;

	my @all_users = AbsenceDB::getUsers();

	#if ((!$AUTH) || (adminAll($user)) || ($op eq 'delete'))
	if ((!$AUTH) || adminAll($user))
	{
		return (@all_users);
	}
	
	# $op is "delete"

	#my $acl = AbsenceDB::getAcl($user);
	#my $self = AbsenceDB::getPidForUser($user);
	my @admgrps = getAdminGroups($user);

	my @uids;
	foreach my $uid (@all_users) {
		@grps = getUserGroups($uid);
		dbg("getAdminUsers: uid=$uid, grps=[".join(',', @grps)."]");
		if (superset(\@admgrps, \@grps)) {
			push(@uids, $uid);
			dbg("pushing [$uid] onto list");
		}
	}

	@uids;
}

#-----------------------------------------------------------------
# return true if all set $gref1 contains all members of set $gref2
#-----------------------------------------------------------------
sub superset
{
	my ($gref1, $gref2) = @_;

	my $count = 0;
	foreach my $mem (@{$gref2}) {
		if (inListN($mem, $gref1)) {
			$count++;
		}
	}

	dbg("superset: count=[$count], len(gref2) = [".@{$gref2}."]");
	($count == @{$gref2}) && return 1;
	return 0;
}

sub getUserGroups
{
	my $uid = shift;

	my $acl = AbsenceDB::getAcl($uid);
	my %uniq_gids;

	foreach my $rule (@{$acl->{g}}) {
		if ($rule =~ /^[war]:(.*)$/) {
			($1 eq 'all') && return (AbsenceDB::getGroups());
			$uniq_gids{$1} = 1;
		}
	}

	return (keys(%uniq_gids));
}

#-----------------------------------------------------------------
# adminForAllGroups() is passed a person-id to check and a list
# of groups for which the user has admin-privs.  It checks if
# the person in question is only a member of groups for which
# the user has admin-privs.  returns true if this is the case,
# false if not.
#-----------------------------------------------------------------
sub adminForAllGroups
{
	my ($pid, $gref) = @_;

	# first get list of groups to which person belongs
	my $pgref = AbsenceDB::getPerson($pid, 'group');
	my $count = 0;
	foreach my $g (@{$pgref}) {
		if (inListN($g, $gref)) {
			$count++;
		}
	}
	if ($count == @{$pgref}) {
		return 1;
	}
	return 0;
}

sub getAdminGroups
{
	my $user = shift;

	my @allgrps = AbsenceDB::getGroups();

	($AUTH && !adminAll($user)) || return (@allgrps);

	my $acl = AbsenceDB::getAcl($user);
	my $self = AbsenceDB::getPidForUser($user);

	my %unique_gids;
	foreach my $rule (@{$acl->{g}}) {
		if ($rule =~ /^a:(.*)$/) {
			my $rhs = $1;
			if ($rhs eq 'self') {
				if ($self) {
					my $gref = AbsenceDB::getPerson($self, 'group');
					foreach my $grp (@{$gref}) {
						$unique_gids{$grp} = 1;
					}
				}
			} else {
				$unique_gids{$rhs} = 1;
			}
		}
	}

	return (keys(%unique_gids));
}

sub divideGroups
{
	my ($user, $uid) = @_;

	my $pid = AbsenceDB::getPidForUser($uid)
		|| die "pid not found for uid [$uid]";
	my $ugref = AbsenceDB::getPerson($pid, 'group');
	my @admgrps = getAdminGroups($user);

	if (adminAll($user)) {
		return ($ugref, ());
	}

	my (@vis, @invis);

	foreach my $grp (@{$ugref}) {
		if (inListN($grp, \@admgrps)) {
			push(@vis, $grp);
		} else {
			push(@invis, $grp);
		}
	}

	(\@vis, \@invis);
}

sub adminAll
{
	my $user = shift;

	$AUTH || return 1;

	my $acl = AbsenceDB::getAcl($user);
	my $g = ','.join(',', @{$acl->{g}}).',';
	return 1 if ($g =~ /,a:all,/);
	return 0;
}

sub getAdminAcls
{
	my ($user, $uid) = @_;

	my $acl = AbsenceDB::getAcl($uid);

	defined($acl) || return({p => [], g => []});

	adminAll($user) && return $acl;

	my ($gvisref, $ginvref) = divideGroupAcls($user, $uid);
	my ($pvisref, $pinvref) = dividePersonAcls($user, $uid);

	return {p => $pvisref, g => $gvisref};
}

sub dividePersonAcls
{
	my ($user, $uid) = @_;

	my $acl = AbsenceDB::getAcl($uid);
	
	my @admppl = getAdminPeople($user);
	my (@vis, @invis);
	foreach my $rule (@{$acl->{p}}) {
		if ($rule =~ /^w:(.*)$/) {
			my ($p) = $1;
			if (($p eq 'self') || inListN($p, \@admppl)) {
				push(@vis, $rule);
			} else {
				push(@invis, $rule);
			}
		}
	}
}

sub selfAllowed
{
	my ($user, $uid) = @_;
	defined($uid) || return 1;
	adminAll($user) && return 1;
	my $acl = AbsenceDB::getAcl($uid);
	my $g = ','.join(',', @{$acl->{g}}).',';
	if ($g =~ /,([war]):self,/) {
		my $pid = AbsenceDB::getPidForUser($uid)
			|| die "pid not found for uid [$uid]";
		my $gref = AbsenceDB::getPerson($pid, 'group');
		my @admgrps = getAdminGroups($user);
		if (superset(\@admgrps, $gref)) {
			return 1;
		}
	}

	return 0;
}

sub deleteElement
{
	my($el, $lref) = @_;

	my $count = 0;
	my $match = 0;
	foreach my $tmp (@$lref) {
		if ($tmp eq $el) {
			$match = 1;
			last;
		}
		$count++;
	}

	if ($match) {
		splice(@$lref, $count, 1);
	}
}

#------------------------------------------------------------------
# divideGroupAcls() is big fun.  It sorts all the ACLs for $uid
# into two groups.  The first group contains the ACLs that $user
# is allowed to administer.  The second groups contains the ACLs
# that $user is not allowed to administer.
#------------------------------------------------------------------
sub divideGroupAcls
{
	my ($user, $uid) = @_;

	my $acl = AbsenceDB::getAcl($uid);
	
	my %porder = (
		a	=> 3,
		w	=> 2,
		r	=> 1,
	);

	my (@vis, @invis);
	my @admgrps = getAdminGroups($user);

	my @rules = @{$acl->{g}};

	my $g = ','.join(',', @{$acl->{g}}).',';
	
	#---------------------------------------------------------------
	# if $uid has an group-ACL containing "self", I must check if
	# $user has admin privs for all groups that $uid is a member of.
	# If $user does not have admin privs for all these groups,
	# the rule containing "self" must be converted into individual
	# rules containing references to real groups.
	#---------------------------------------------------------------
	if ($g =~ /,([war]):self,/) {
		my $perm = $1;
		my $pid = AbsenceDB::getPidForUser($uid)
			|| die "pid not found for uid [$uid]";
		my $gref = AbsenceDB::getPerson($pid, 'group');
		if (!superset(\@admgrps, $gref)) {
			deleteElement("$perm:self", \@rules);
			foreach my $grp (@{$gref}) {
				if ($g =~ /,([war]):$grp,/) {
					my $p = $1;
					if ($porder{$p} < $porder{$perm}) {
						# take new one
						push(@rules, "$perm:$grp");
						deleteElement("$p:$grp", \@rules);
					}
				} else {
					push(@rules, "$perm:$grp");
				}
			}
		}
		dbg("divideGroupAcls: after self subst: [".join(',', @rules).']');
	}

	#---------------------------------------------------------------
	# if there is a rule containing "all", I must do the same thing
	# as described above for "self".
	#---------------------------------------------------------------
	$g = ','.join(',', @rules).',';
	if ($g =~ /,([war]):all,/) {
		my $perm = $1;
		if ($perm =~ /^[rw]$/) {
			deleteElement("$perm:all", \@rules);
			foreach my $grp (AbsenceDB::getGroups()) {
				if ($g =~ /,([war]):$grp,/) {
					my $p = $1;
					if ($porder{$p} < $porder{$perm}) {
						# take new one
						push(@rules, "$perm:$grp");
						deleteElement("$p:$grp", \@rules);
					}
				} else {
					push(@rules, "$perm:$grp");
				}
			}
		}
		dbg('divideGroupAcls: after all subst: ['.join(',', @rules).']');
	}

	#---------------------------------------------------------------
	# now I can go through my list of rules and divide into the
	# two groups.  If a rule refers to a group for which $user has
	# admin privs, it goes on the visable list, otherwise on the
	# invisible list.
	#---------------------------------------------------------------
	foreach my $rule (@rules) {
		if ($rule =~ /^([war]):(.*)$/) {
			my ($p, $g) = ($1, $2);
			if ($g =~ /^(self|all)$/) {
				if ($g eq 'self') {
					push(@vis, $rule);
				} else {
					push(@invis, $rule);
				}
				next;
			}
			if (inListN($g, \@admgrps)) {
				push(@vis, $rule);
			} else {
				push(@invis, $rule);
			}
		}
	}

	return (\@vis, \@invis);
}

sub inListN
{
	my ($thing, $lref) = @_;

	foreach my $elem (@{$lref}) {
		($elem == $thing) && return 1;
	}

	return 0;
}

1;
