#!/usr/bin/perl

#=====================================================================
# Welcome to "absence-manage.pl"!
#
# absence-manage.pl is somewhat complicated because it is called at
# two stages (see also the comments at the top of absence-click.pl).
#
# Stage 1:
#
# user clicks on the "manage" URL in the control frame and is presented
# with a number of forms for managing people and groups.
#
# Stage 2:
#
# user clicks on one of the submit buttons belonging to one of the
# forms above and the script is re-entered with some CGI parameters
# set.
#=====================================================================

use CGI qw/:standard -debug/;
# uncomment the following if you are debugging
use CGI::Carp qw(fatalsToBrowser);
use FileHandle;

BEGIN {
	if ($ENV{QUERY_STRING} =~ /^instance=(\S+)$/) {
		# an instance has been specified
		$ENV{INSTANCE_NAME} = $1;
	}
}

use AbsenceDB;
use AbsenceConfig;
use AbsenceLog;
use AbsenceAuthentication;
use AbsenceAuthorization;

#--------------------------------------------------------------------
# global variables
#--------------------------------------------------------------------
my $IMAGE_DIR		= AbsenceConfig::fetch('image_dir_abs');
my $TOP_PAGE		= AbsenceConfig::fetch('top_page');
my $MAIN_SCRIPT		= AbsenceConfig::fetch('main_script');
my $GROUP_POLICY	= AbsenceConfig::fetch('group_policy');
my $OAP				= AbsenceConfig::fetch('objects_are_people');
my $AUTH			= AbsenceConfig::fetch('authentication');
#my $CRED_SRC		= AbsenceConfig::fetch('credential_src');
my $MANAGE_PASSWORD	= AbsenceConfig::fetch('manage_password');

#my $DEBUG_OUT		= '/tmp/manage.log';
my $VERSION		= '1.5';
my $DEBUG			= 0;

$CGI::POST_MAX = 1000;
$CGI::DISABLE_UPLOADS = 1;

#--------------------------------------------------------------------
# main()
#--------------------------------------------------------------------
if (exists($ENV{INSTANCE_NAME})) {
	$MAIN_SCRIPT .= "?instance=$ENV{INSTANCE_NAME}";
}

my $q = new CGI;
$q->import_names;

#-----------------------------------------------------------------
# first, authenticate if necessary
#-----------------------------------------------------------------
my $AUTH_USER = AbsenceAuthentication::checkAuthentication($q);

#-----------------------------------------------------------------
# check if $AUTH_USER is allowed to perform any admin tasks. If not,
# complain.
#-----------------------------------------------------------------
if ($AUTH
	&& !AbsenceAuthorization::getAdminGroups($AUTH_USER)
	&& defined($Q::action)
	&& ($Q::action ne 'Change Password'))
{
	print $q->header();
	print $q->start_html(
		-title		=> 'Absence: Manage People',
		-target		=> 'display',
		-BGCOLOR	=> '#AAA0D8',
	),
	"<H3>You do not have any admin priviledges.</H3>\n",
	$q->end_html();
	exit;
}


# DEBUG
if ($DEBUG) {
	my @keywords = $q->keywords;
	my @names = $q->param;
	dbg('Parameters:');
	foreach my $p (@names) {
		dbg(" [$p] ".$q->param($p));
	}

	dbg("Keywords:");
	foreach my $p (@keywords) {
		dbg(" [$p] ");
	}
}


if (defined($Q::action)) {
	if ($Q::action eq 'Add Person') {
		addPerson($q);
	} elsif ($Q::action eq 'Delete Person') {
		deletePerson($q);
	} elsif ($Q::action eq 'Modify Person') {
		modifyPersonForm($q);
	} elsif ($Q::action eq 'Add User') {
		addUser($q);
	} elsif ($Q::action eq 'Modify User') {
		modifyUserForm($q);
	} elsif ($Q::action eq 'Delete User') {
		deleteUser($q);
	} elsif ($Q::action eq 'Modify Group') {
		modifyGroupForm($q);
	} elsif ($Q::action eq 'Delete Group') {
		deleteGroup($q);
	} elsif ($Q::action eq 'Add Group') {
		addGroup($q);
	} elsif ($Q::action eq 'Submit Modification') {
		dbg("got [Submit Modification]");
		modifyPerson($q);
	} elsif ($Q::action eq 'Submit Group Modification') {
		modifyGroup($q);
	} elsif ($Q::action eq 'Submit User Modification') {
		modifyUser($q);
	} elsif ($Q::action eq 'Change Password') {
		changePassword($q);
	}
}

print $q->header(-expires => '-1d');
print $q->start_html(
	-title		=> 'Absence: Manage People',
	-target		=> 'display',
	-BGCOLOR	=> '#AAA0D8',
);

# include JavaScript for ACL-managment if necessary
if ($AUTH) {
	includeJsAclScript();
}

#print "<H2>AUTH = [$AUTH]</H2>\n";

($AUTH && $MANAGE_PASSWORD) && changePasswordForm();
if ($AUTH && !AbsenceAuthorization::getAdminGroups($AUTH_USER)) {
	$q->end_html();
	exit;
}

print "<HR><P>\n";
print $q->h1({-align => 'center'},'Manage People, Groups &amp; Users');

print "<HR>\n<P>\n";

mainAddPersonForm($q);
mainModifyPersonForm($q);
mainDeletePersonForm($q);

if (!$AUTH || AbsenceAuthorization::adminAll($AUTH_USER)) {
	mainAddGroupForm($q);
	mainModifyGroupForm($q);
	mainDeleteGroupForm($q);
}

if ($AUTH) {
	mainAddUserForm($q);
	mainModifyUserForm($q);
	mainDeleteUserForm($q);
}


print $q->start_form(-action => $MAIN_SCRIPT, -method => 'POST');
print $q->submit(action => 'Cancel');
print $q->end_form();
print $q->end_html();

exit;

#=================================================================
#=================================================================
# first level forms
#=================================================================
#=================================================================

sub changePasswordForm
{
	print $q->h1('Change Password for '.$AUTH_USER);
	print $q->start_form(
		-name		=> 'change_password',
	);
	print "<TABLE>\n";
	print "<TR><TH ALIGN=right>New Password</TH><TD>\n";
	print $q->password_field(
		-name		=> 'password1',
		-size		=> 30,
		-maxlength	=> 30,
	);
	print "</TD></TR>\n";
	print "<TR><TH ALIGN=right>New Password (again)</TH><TD>\n";
	print $q->password_field(
		-name		=> 'password2',
		-size		=> 30,
		-maxlength	=> 30,
	);
	print "</TD></TR>\n</TABLE>\n";
	print $q->submit(action => 'Change Password');
	print $q->end_form();
}

sub changePassword
{
	my $q = shift;

	if (!length($Q::password1) || !length($Q::password2)) {
		dbg("looks bad");
		reloadDisplay($q, 'Error changing password',
			'one or both password fields blank');
	}

	if ($Q::password1 ne $Q::password2) {
		dbg("looks bad");
		reloadDisplay($q, 'Error changing password', 'password mismatch');
	}

	my $ret = AbsenceDB::setPassword($AUTH_USER, $Q::password1);
	dbg("ret = [$ret]");

	if ($ret eq 'ok') {
		dbg("looks good");
		print $q->redirect($MAIN_SCRIPT);
	} elsif ($ret eq 'disappeared') {
		dbg("looks bad");
		reloadDisplay($q, 'Error changing password',
			"user [$AUTH_USER] disappeared");
	} elsif ($ret eq 'error') {
		dbg("looks bad");
		reloadDisplay($q, 'Error changing password', 'error changing password');
	}

	exit;
}

#---------------------------------------------------------------
# ADD PERSON
#---------------------------------------------------------------

sub mainAddPersonForm
{
	my $q = shift;

	print $q->h1('Add a person');
	print qq[<P>A "person" is an visible object which can have absences
		assigned to it.<P>];
	my @onsubmit = ($OAP && $AUTH)
		? (-onSubmit   => 'return validate(document.add_person);') 
		: ();
	print $q->start_form(
		-name		=> 'add_person',
		@onsubmit,
	);
	print "<TABLE>\n";

	#------------------------
	# Name
	#------------------------
	print "<TR><TH ALIGN=right>Name (Last, First)</TH><TD>\n";
	print $q->textfield(
		-name		=> 'name',
		-size		=> 20,
		-maxlength	=> 20,
	);
	print "</TD><TD>[This will appear in the month-images]</TD></TR>\n";

	if ($AUTH && $OAP) {
		#------------------------
		# User-ID
		#------------------------
		print "<TR><TH ALIGN=right>User-ID</TH><TD>\n";
		print $q->textfield(
			-name		=> 'user_id',
			-size		=> 20,
			-maxlength	=> 20,
		);
		print "</TD><TD>[This is the login-name]</TD></TR>\n";
		#------------------------
		# password
		#------------------------
		if ($MANAGE_PASSWORD) {
			print "<TR><TH ALIGN=right>Password</TH><TD>\n";
			print $q->textfield(
				-name		=> 'password',
				-size		=> 30,
				-maxlength	=> 30,
			);
			print "</TD></TR>\n";
		}
	}

	#------------------------
	# E-mail
	#------------------------
	print "<TR><TH ALIGN=right>E-Mail Address</TH><TD>\n";
	print $q->textfield(
		-name		=> 'email',
		-size		=> 30,
		-maxlength	=> 50,
	);

	#------------------------
	# groups
	#------------------------
	print "<TR><TH ALIGN=right VALIGN=top>Group</TH><TD>\n";
	($GROUP_POLICY eq 'single')
		? makeGroupPopup($q)
		: makeGroupCheckboxes($q, []);

	print "</TD>
		<TD VALIGN=top>
			[choose which groups this person should belong to]
		</TD></TR>\n";

	($AUTH && $OAP) && aclForm($q, 'add_person');

	print "</TABLE>\n";
	print $q->submit(action => 'Add Person');
	print $q->end_form();
	print "<HR>\n";
}

#---------------------------------------------------------------
# MODIFY PERSON
#---------------------------------------------------------------

sub mainModifyPersonForm
{
	my $q = shift;

	print $q->h1('Modify a person');
	print $q->start_form;
	print "Person";
	makePeoplePopup($q);
	print "&nbsp;";
	print $q->submit(action => 'Modify Person');
	print $q->end_form();
	print "<HR>\n";
}

#---------------------------------------------------------------
# DELETE PERSON
#---------------------------------------------------------------

sub mainDeletePersonForm
{
	my $q = shift;

	print $q->h1('Delete a person');
	print $q->start_form;
	print "Person";
	makePeoplePopup($q, 'delete');

	print "&nbsp;";
	print $q->submit(action => 'Delete Person');
	print $q->end_form();

	print "<HR>\n";
}

#---------------------------------------------------------------
# ADD GROUP
#---------------------------------------------------------------

sub mainAddGroupForm
{
	my $q = shift;

	print $q->h1('Add a Group');
	print qq[<P>A "group" is a collection of "people".<P>];
	print $q->start_form;
	print "Name\n";
	print $q->textfield(
		-name		=> 'name',
		-size		=> 20,
		-maxlength	=> 20,
	);
	print "&nbsp;";

	print $q->submit(action => 'Add Group');
	print $q->end_form();
}

#---------------------------------------------------------------
# MODIFY GROUP
#---------------------------------------------------------------

sub mainModifyGroupForm
{
	my $q = shift;

	print "<HR>\n";

	print $q->h1('Modify a group');
	print $q->start_form(-name => 'ModifyGroup');
	print "Group";
	makeGroupPopup($q);
	print "&nbsp;";
	print $q->submit(action => 'Modify Group');
	print $q->end_form();

	print "<HR>\n";
}

#---------------------------------------------------------------
# DELETE GROUP
#---------------------------------------------------------------

sub mainDeleteGroupForm
{
	my $q = shift;

	print $q->h1('Delete a group');
	if ($GROUP_POLICY eq 'single') {
		print "(this will also delete <EM>all</EM> members of the group)<BR>\n";
	} else {
		print "(this will also delete <EM>all</EM> members of the group\n";
		print "who are in no other groups)<BR>\n";
	}
	print $q->start_form(-name => 'DeleteGroup');
	print "Group";
	makeGroupPopup($q);
	print "&nbsp;";
	print $q->submit(action => 'Delete Group');
	print $q->end_form();

	print "<HR>\n";
}

#---------------------------------------------------------------
# ADD USER
#---------------------------------------------------------------

sub mainAddUserForm
{
	my $q = shift;

	print $q->h1('Add a user');
	print qq[<P>A "user" is purely an authentication object, with no
		person-object associated with it.<P>];

	# ROBBO
	print $q->start_form(
		-name		=> 'add_user',
		-onSubmit   => 'return validate(document.add_user);',
	);
	print "<TABLE>\n";

	#------------------------
	# User-ID
	#------------------------
	print "<TR><TH ALIGN=right>User-ID</TH><TD>\n";
	print $q->textfield(
		-name		=> 'user_id',
		-size		=> 20,
		-maxlength	=> 20,
	);
	print "</TD><TD>[This is the login-name]</TD></TR>\n";


	if ($MANAGE_PASSWORD) {
		#------------------------
		# password
		#------------------------
		if ($MANAGE_PASSWORD) {
			print "<TR><TH ALIGN=right>Password</TH><TD>\n";
			print $q->textfield(
				-name		=> 'password',
				-size		=> 30,
				-maxlength	=> 30,
			);
			print "</TD></TR>\n";
		}
	}

	aclForm($q, 'add_user');

	print "</TABLE>\n";
	print $q->submit(action => 'Add User');
	print $q->end_form();
	print "<HR>\n";
}

sub addUser
{
	my $q = shift;

	dbg("- add user -");

	if (!length($Q::user_id)) {
		dbg("looks bad");
		reloadDisplay($q, 'Error adding person', 'You must specify a User-ID.');
	}

	if ($Q::user_id eq 'self') {
		dbg("looks bad");
		reloadDisplay($q, 'Error adding person', 'User-ID must not be "self".');
	}

	if ($Q::user_id =~ /\s/) {
		dbg("looks bad");
		reloadDisplay($q, 'Error adding person',
			'User-ID must not have any spaces or tabs in it.');
	}

	if ($MANAGE_PASSWORD && !length($Q::password)) {
		dbg("looks bad: no password supplied");
		reloadDisplay($q, 'Error adding person', 'You must specify a Password.');
	}


	my ($pacl, $gacl) = convertAcl(\@Q::person_acl, \@Q::group_acl);
	if (!checkAcls($pacl, $gacl, 'add-user')) {
		dbg("looks bad: acl violation");
		reloadDisplay($q, 'Error adding person', 'Invalid person/groups');
	}

	my @params = ($Q::user_id, $Q::password, $pacl, $gacl);

	dbg("params=(".join(',',@params).")");
	my $ret = AbsenceDB::addUser(@params);
	dbg("ret = [$ret]");

	if ($ret eq 'ok') {
		dbg("looks good");
		print $q->redirect($MAIN_SCRIPT);
	} elsif ($ret eq 'duplicate') {
		dbg("looks bad");
		reloadDisplay($q, 'Error adding user',
			"user-ID [$Q::user_id] already exists in system");
	}

	exit;
}

sub mainModifyUserForm
{
	my $q = shift;

	print $q->h1('Modify a user');
	print $q->start_form;
	print "User";
	makeUsersPopup($q, 'modify');
	print "&nbsp;";
	print $q->submit(action => 'Modify User');
	print $q->end_form();
	print "<HR>\n";
}

sub modifyUserForm
{
	my $q = shift;

	my $uref = AbsenceDB::getUser($Q::user_id);

	if (!defined($uref)) {
		reloadDisplay($q, 'Error modifying user', 'User disappeared.');
	}

	print $q->header(-expires => '-1d');
	print $q->start_html(
		-title		=> "Absence: Modify $Q::user_id",
		-target		=> 'display',
		-BGCOLOR	=> '#AAA0D8',
	);

	includeJsAclScript();
	print $q->h1("Modify $Q::user_id");

	print $q->start_form(
		-method		=> 'POST',
		-name		=> 'mod_user',
		-onSubmit	=> 'return validate(document.mod_user);',
	);

	print "<TABLE>\n";

	#---------------------------------------------
	# User-ID
	#---------------------------------------------
	print qq[<TR><TH ALIGN=right>User-ID</TH><TD BGCOLOR="#bbbbbb">\n];
	print $q->hidden(-name => 'user_id', -default => $Q::user_id);
	print "$Q::user_id";
	print "</TD>\n";
	print "<TD>[login-name]</TD>\n";
	print "</TR>\n";

	#------------------------
	# password
	#------------------------
	if ($MANAGE_PASSWORD) {
		print "<TR><TH ALIGN=right>Password</TH><TD>\n";
		print $q->textfield(
			-name		=> 'password',
			-size		=> 30,
			-maxlength	=> 30,
			-value		=> $uref->{password},
		);
		print "</TD><TD>[to change password, clear and enter new password]</TR>\n";
	}

	aclForm($q, 'mod_user', $Q::user_id);

	print "</TABLE>\n";

	print $q->submit(action => 'Submit User Modification');
	print $q->end_form();
	print $q->end_html();
	exit;
}

sub modifyUser
{
	my $q = shift;

	dbg("- modify user -");

	if (!length($Q::user_id)) {
		dbg("looks bad");
		reloadDisplay($q, 'Error modifying user',
			'User-ID field must not be blank.');

		if ($MANAGE_PASSWORD && !length($Q::password)) {
			dbg("looks bad: no password supplied");
			reloadDisplay($q, 'Error adding user',
				'You must specify a Password.');
		}
	}

	my $oldinfo = AbsenceDB::getUser($Q::user_id);

	if (!defined($oldinfo)) {
		reloadDisplay($q, 'Error modifying user',
			"user [$Q::user_id] disappeared.");
	}

	dbg("pacl=[".join(',',@Q::person_acl)."]");
	dbg("gacl=[".join(',',@Q::group_acl)."]");
	my ($pacl, $gacl) = convertAcl(\@Q::person_acl, \@Q::group_acl);
	dbg("(converted) pacl=[".join(',',@{$pacl})."]");
	dbg("(converted) gacl=[".join(',',@{$gacl})."]");
	if (!checkAcls($pacl, $gacl, 'mod-user', $Q::user_id)) {
		dbg("looks bad: acl violation");
		reloadDisplay($q, 'Error adding user', 'Invalid person/groups');
	}

	recombineAcls($AUTH_USER, $Q::user_id, $pacl, $gacl);
	my @params = ($Q::user_id, $Q::password, $pacl, $gacl);

	my $ret = AbsenceDB::modifyUser(@params);

	dbg("ret = [$ret]");

	if ($ret eq 'ok') {
		dbg("looks good");
		print $q->redirect($MAIN_SCRIPT);
		exit;
	} else {
		dbg("looks bad");
		abslog("error modifying user_id [$Q::user_id] ret = [$ret]");
		reloadDisplay($q, 'Error modifying user',
			"modify-user [$Q::user_id] failed.");
	}
}

sub mainDeleteUserForm
{
	my $q = shift;

	print $q->h1('Delete a user');
	print $q->start_form;
	print "User";
	makeUsersPopup($q, 'delete');

	print "&nbsp;";
	print $q->submit(action => 'Delete User');
	print $q->end_form();

	print "<HR>\n";
}

# ROBBO
sub deleteUser
{
	my $q = shift;

	my $ret = AbsenceDB::deleteUser($Q::user_id);
	if ($ret eq 'ok') {
		print $q->redirect($MAIN_SCRIPT);
	} elsif ($ret eq 'bad-id') {
		reloadDisplay($q, 'Error deleting user',
			'User was deleted by someone else');
	}
	exit;
}

#=================================================================
#=================================================================
#=================================================================

sub makeUsersPopup
{
	my ($q, $op) = @_;

	my @users = AbsenceAuthorization::getAdminUsers($AUTH_USER, $op);

	print $q->popup_menu(
		-name   => 'user_id',
		-values => [sort @users],
	);
}

sub makePeoplePopup
{
	my ($q, $op) = @_;

	my @people = AbsenceAuthorization::getAdminPeople($AUTH_USER, $op);

	my $labref;
	foreach my $pid (@people) {
		$labref->{$pid} = AbsenceDB::getPerson($pid, 'name');
	}

	my @sorted = sort {lc($labref->{$a}) cmp lc($labref->{$b})} @people;

	print $q->popup_menu(
		-name   => 'person_id',
		-values => \@sorted,
		-labels	=> $labref,
	);
}

sub removeMonthImages
{
	my (@list) = @_;

	local(*DIR);
	opendir(DIR, $IMAGE_DIR) || die "opendir [$IMAGE_DIR]";
	
	my $it = AbsenceConfig::fetch('image_type');
	my $file;
	my $re = '('.join('|',@list).')';
	while($_ = readdir(DIR)) {
		if ((@list && /^absence-${re}-.*\.$it$/) ||
			(!@list && /^absence-.*\.$it$/))
		{
			$file = "$IMAGE_DIR/$_";
			dbg("unlinking [$file]");
			unlink($file) || dbg("failed: $!");
		}
	}
	closedir(DIR);
}

#=================================================================
# add a group
#=================================================================

sub addGroup
{
	my $q = shift;

	dbg("- add group -");

	if (!AbsenceAuthorization::adminAll($AUTH_USER)) {
		abslog("attempt to circumvent security (addGroup)");
		reloadDisplay($q, 'Error adding group', 'security breach thwarted');
	}

	if (length($Q::name) == 0) {
		dbg("looks bad");
		reloadDisplay($q, 'Error adding group', 'You must supply a name.');
	}

	if ($Q::name =~ /^(all|self)$/i) {
		dbg("looks bad");
		reloadDisplay($q, 'Error adding group', 'The name must not be "self" or "all".');
	}

	my $ret = AbsenceDB::addGroup('new', $Q::name, $Q::pass);
	dbg("ret = [$ret]");

	if ($ret eq 'ok') {
		dbg("looks good");
		reloadTop($q, 'Group Successfully Added', '_top',
			'Finished. Click on button below to reload top page');
	} elsif ($ret eq 'duplicate') {
		dbg("looks bad");
		reloadDisplay($q, 'Error adding group',
			"[$Q::name] already exists in system");
	} else {
		dbg("looks bad");
		reloadDisplay($q, 'Error adding group',
			"[$Q::name] was deleted by someone else");
	}
	exit;
}

sub deleteGroup
{
	my $q = shift;

	dbg("- delete group -");

	if (!AbsenceAuthorization::adminAll($AUTH_USER)) {
		abslog("attempt to circumvent security (deleteGroup)");
		reloadDisplay($q, 'Error deleting group', 'security breach thwarted');
	}

	my $ret = AbsenceDB::deleteGroup($Q::group_id);
	dbg("ret = [$ret]");

	if ($ret eq 'ok') {
		dbg("looks good");
		reloadTop($q, 'Group Successfully Deleted', '_top',
			'Finished. Click on button below to reload top page');
		removeMonthImages($Q::group_id);
	} else {
		dbg("looks bad");
		reloadTop($q, 'Error deleting group', 'display',
			'someone deleted the group before you.');
	}
	exit;
}

#=================================================================
# add a person
#=================================================================

sub addPerson
{
	my $q = shift;

	dbg("- add person -");

	if (length($Q::name) == 0) {
		dbg("looks bad");
		reloadDisplay($q, 'Error adding person', 'You must supply a name.');
	}

	if ($Q::name eq 'self') {
		dbg("looks bad");
		reloadDisplay($q, 'Error adding person', 'Name must not be "self".');
	}

	if ($AUTH && $OAP && !length($Q::user_id)) {
		dbg("looks bad");
		reloadDisplay($q, 'Error adding person', 'You must specify a User-ID.');

		if ($MANAGE_PASSWORD && !length($Q::password)) {
			dbg("looks bad: no password supplied");
			reloadDisplay($q, 'Error adding person',
				'You must specify a Password.');
		}
	}

	my @grps = ($GROUP_POLICY eq 'single') ? ($Q::group_id) : @Q::groups;

	if (!checkGroups(@grps)) {
		dbg("looks bad: acl violation");
		reloadDisplay($q, 'Error adding person', 'Invalid group membership');
	}

	my @params = ('new', $Q::name, $Q::user_id, $Q::email, \@grps);
	if ($OAP && $AUTH) {
		my ($pacl, $gacl) = convertAcl(\@Q::person_acl, \@Q::group_acl);
		if (!checkAcls($pacl, $gacl, 'add-person')) {
			dbg("looks bad: acl violation");
			reloadDisplay($q, 'Error adding person', 'Invalid person/groups');
		}
		push(@params, $Q::password, $pacl, $gacl);
	}

	dbg("params=(".join(',',@params).")");
	my $ret = AbsenceDB::addPerson(@params);
	dbg("ret = [$ret]");

	if ($ret eq 'ok') {
		dbg("looks good");
		print $q->redirect($MAIN_SCRIPT);
		foreach my $gid (@grps) {
			removeMonthImages($gid);
		}
	} elsif ($ret =~ /^duplicate/) {
		dbg("looks bad");
		if ($ret eq 'duplicate-name') {
			reloadDisplay($q, 'Error adding person',
				"[$Q::name] already exists in system");
		} else {
			reloadDisplay($q, 'Error adding person',
				"user-ID [$Q::user_id] already exists in system");
		}
	}
	exit;
}

sub recombineAcls
{
	my ($user, $uid, $pacl, $gacl) = @_;

	if (AbsenceAuthorization::adminAll($AUTH_USER)) {
		return ($pacl, $gacl);
	}

	my ($gvisref, $ginvref, $pvisref, $pinvref) = ([], [], [], []);
	my $uref = AbsenceDB::getUser($Q::uid);
	if (defined($uref)) {
	 ($gvisref, $ginvref) = AbsenceAuthorization::divideGroupAcls($user, $uid);
	 ($pvisref, $pinvref) = AbsenceAuthorization::dividePersonAcls($user, $uid);
	}

	push(@{$pacl}, @{$pinvref});
	push(@{$gacl}, @{$ginvref});

}

sub checkGroups
{
	my @grps = @_;

	$AUTH || return 1;

	my @admgroups = AbsenceAuthorization::getAdminGroups($AUTH_USER);
	foreach my $grp (@grps) {
		if (!inListN($grp, \@admgroups)) {
			return 0;
		}
	}

	return 1;
}

sub checkAcls
{
	$AUTH || return 1;

	my ($paclr, $gaclr, $op, $uid) = @_;
	my @admgroups = AbsenceAuthorization::getAdminGroups($AUTH_USER);
	my @admpeople = AbsenceAuthorization::getAdminPeople($AUTH_USER);

	dbg("checkAcls: people");
	foreach my $rule (@{$paclr}) {
		my ($perm, $pid) = ($rule =~ /^(r|w|a):(.*)$/);
		dbg("rule=[$rule], perm=[$perm], gid=[$pid]");
		if ($pid eq 'self') { next; }
		if (!inListN($pid, \@admpeople)) {
			dbg("did not find [$pid] in (".join(',', @admpeople).")");
			return 0;
		}
	}

	my $all = AbsenceAuthorization::adminAll($AUTH_USER);
	dbg("checkAcls: groups");
	foreach my $rule (@{$gaclr}) {
		my ($perm, $gid) = ($rule =~ /^(r|w|a):(.*)$/);
		dbg("rule=[$rule], perm=[$perm], gid=[$gid]");
		if ($gid eq 'self') {
			($op eq 'mod-person') &&
				!AbsenceAuthorization::selfAllowed($AUTH_USER, $uid) &&
					return 0;
			next;
		}
		if ($gid eq 'all') {
			if ($all) { next; }
			return 0;
		}
		if (!inListN($gid, \@admgroups)) {
			dbg("did not find [$gid] in (".join(',', @admgroups).")");
			return 0;
		}
	}

	return 1;
}

sub convertAcl
{
	my @res;
	for(my $ind = 0; $ind < 2; $ind++) {
		foreach my $rule (@{$_[$ind]}) {
			if ($rule =~ /^([^:]+):(read|write|admin)$/) {
				my ($gid, $perm) = ($1, substr($2, 0, 1));
				push(@{$res[$ind]}, "${perm}:$gid");
			} else {
				die "unable to match [$rule]";
			}
		}
	}

	@res;
}

sub modifyPerson
{
	my $q = shift;

	dbg("- modify person -");
	if (length($Q::name) == 0) {
		dbg("looks bad");
		reloadDisplay($q, 'Error modifying person', 'You must supply a name.');
	}

	if ($AUTH && $OAP && !length($Q::user_id)) {
		dbg("looks bad");
		reloadDisplay($q, 'Error modifying person',
			'User-ID field must not be blank.');

		if ($MANAGE_PASSWORD && !length($Q::password)) {
			dbg("looks bad: no password supplied");
			reloadDisplay($q, 'Error adding person',
				'You must specify a Password.');
		}
	}

	my @grps = ($GROUP_POLICY eq 'single') ? ($Q::group_id) : @Q::groups;

	if (!checkGroups(@grps)) {
		dbg("looks bad: acl violation");
		reloadDisplay($q, 'Error adding person', 'Invalid group membership');
	}

	dbg("Q::groups=[".join(',', @grps)."]");
	my $oldinfo = AbsenceDB::getPerson($Q::person_id);
	if (!defined($oldinfo)) {
		reloadDisplay($q, 'Error modifying person',
			"person [$Q::name] disappeared.");
	}

	dbg("modifyPerson: modified groups=[".join(',', @grps)."]");
	# recombine visible and invisible groups
	my $uref = AbsenceDB::getUser($Q::user_id);
	my $inr = [];
	if (defined($uref)) {
		$inr = (AbsenceAuthorization::divideGroups($AUTH_USER, $Q::user_id))[1];
	}
	dbg("modifyPerson: invisible groups=[".join(',', @{$inr})."]");
	push(@grps, @{$inr});
	dbg("modifyPerson: recombined groups=[".join(',', @grps)."]");

	my @params = ($Q::person_id, $Q::name, $Q::user_id, $Q::email, \@grps);
	if ($OAP && $AUTH) {
		dbg("pacl=[".join(',',@Q::person_acl)."]");
		dbg("gacl=[".join(',',@Q::group_acl)."]");
		my ($pacl, $gacl) = convertAcl(\@Q::person_acl, \@Q::group_acl);
		dbg("(converted) pacl=[".join(',',@{$pacl})."]");
		dbg("(converted) gacl=[".join(',',@{$gacl})."]");
		if (!checkAcls($pacl, $gacl, 'mod-person', $Q::user_id)) {
			dbg("looks bad: acl violation");
			reloadDisplay($q, 'Error adding person', 'Invalid person/groups');
		}
		recombineAcls($AUTH_USER, $Q::user_id, $pacl, $gacl);
		push(@params, $Q::password, $pacl, $gacl);
		dumpAcl("modifyPerson: uid=$Q::user_id, RECOMBINED:",
			{p => $pacl, g => $gacl});
	}

	my @old_groups = @{$oldinfo->{group}};
	my $ret = AbsenceDB::addPerson(@params);

	dbg("ret = [$ret]");

	if ($ret eq 'ok') {
		dbg("looks good");
		print $q->redirect($MAIN_SCRIPT);
		my (@new, @old);
		@old = @old_groups;
		foreach my $g (@old_groups) {
			if (inListN($g, \@grps)) {
				deleteElementN($g, \@old);
			} else {
				push(@new, $g);
			}
		}
		push(@new, @old);

		removeMonthImages(@new);

		exit;
	} else {
		dbg("looks bad");
		abslog("error modifying person_id [$Q::person_id] ret = [$ret]");
		reloadDisplay($q, 'Error modifying person',
			"modify-person [$Q::name] failed.");
	}
}

sub deleteElementN
{
	my($elem, $lref) = @_;

	my $count = 0;
	my $match = 0;
	foreach my $tmp (@$lref) {
		if ($tmp == $elem) {
			$match = 1;
			last;
		}
		$count++;
	}

	if ($match) {
		splice(@$lref, $count, 1);
	}
}

sub inListN
{
	my ($thing, $lref) = @_;

	foreach my $elem (@{$lref}) {
		($thing == $elem) && return 1;
	}

	return 0;
}

sub modifyPersonForm
{
	my $q = shift;

	my $pref = AbsenceDB::getPerson($Q::person_id);

	dbg("modifyPersonForm: person_id=[$Q::person_id]");
	print $q->header(-expires => '-1d');
	print $q->start_html(
		-title		=> "Absence: Modify $pref->{name}",
		-target		=> 'display',
		-BGCOLOR	=> '#AAA0D8',
	);

	if (!defined($pref)) {
		reloadDisplay($q, 'Error modifying person', 'Person disappeared.');
	}

	if ($AUTH) {
		includeJsAclScript();
	}
	print $q->h1("Modify $pref->{name}");

	my @onsubmit = ($OAP && $AUTH)
		? (-onSubmit   => 'return validate(document.mod_person);') 
		: ();

	print $q->start_form(
		-method	=> 'POST',
		-name	=> 'mod_person',
		@onsubmit,
	);
	print $q->hidden(-name => 'person_id', -default => $Q::person_id);
	print "<TABLE>\n";

	#---------------------------------------------
	# name
	#---------------------------------------------
	print "<TR><TH ALIGN=right>Name (Last, First)</TH><TD>\n";
	print $q->textfield(
		-name		=> 'name',
		-size		=> 20,
		-maxlength	=> 20,
		-value		=> $pref->{name},
	);
	print "</TD><TD>[This will appear in the month-images]</TD></TR>\n";

	#---------------------------------------------
	# User-ID
	#---------------------------------------------
	my $uid;
	if ($AUTH && $OAP) {
		$uid = exists($pref->{user_id}) ? $pref->{user_id} : undef;
		if ($uid && !AbsenceDB::getUser($uid)) { $uid = undef; }
		print "<TR><TH ALIGN=right>User-ID</TH><TD>\n";
		print $q->textfield(
			-name		=> 'user_id',
			-size		=> 20,
			-maxlength	=> 20,
			-value		=> $uid,
		);
		print "</TD><TD>[This is the login-name</TD></TR>\n";
		#------------------------
		# password
		#------------------------
		my $pw = '';
		if ($uid) {
			$pw  = AbsenceDB::getUser($pref->{user_id}, 'password');
			if (!defined($pw)) { $pw = ''; }
		}
		if ($MANAGE_PASSWORD) {
			print "<TR><TH ALIGN=right>Password</TH><TD>\n";
			print $q->textfield(
				-name		=> 'password',
				-size		=> 30,
				-maxlength	=> 30,
				-value		=> $pw,
			);
			print "</TD><TD>[to change password, clear and enter new password]</TD></TR>\n";
		}
	}

	#---------------------------------------------
	# E-Mail address
	#---------------------------------------------
	print "<TR><TH ALIGN=right>E-Mail Address</TH><TD>\n";
	print $q->textfield(
		-name		=> 'email',
		-size		=> 30,
		-maxlength	=> 50,
		-value		=> exists($pref->{email}) ? $pref->{email} : '',
	);
	print "</TD></TR>\n";

	#---------------------------------------------
	# group
	#---------------------------------------------
	print "<TR><TH ALIGN=right VALIGN=top>Group</TH><TD>\n";

	#abslog("calling makeGroupPopup with gid=$gid");

	if ($GROUP_POLICY eq 'single') {
		makeGroupPopup($q, $pref->{group}->[0]);
	} else {
		makeGroupCheckboxes($q, $pref->{group});
	}

	print "</TD><TD VALIGN=top>[select groups to which person should belong]</TD></TR>\n";

	($AUTH && $OAP) && aclForm($q, 'mod_person', $uid);
	print "</TABLE>\n";

	print $q->submit(action => 'Submit Modification');
	print $q->end_form();
	print $q->end_html();
	exit;
}

sub modifyGroup
{
	my $q = shift;

	if (!AbsenceAuthorization::adminAll($AUTH_USER)) {
		abslog("attempt to circumvent security (modifyGroup)");
		reloadDisplay($q, 'Error modifying group', 'security breach thwarted');
	}

	dbg("- modify group -");
	if (length($Q::name) == 0) {
		dbg("looks bad");
		reloadDisplay($q, 'Error modifying group', 'You must supply a name.');
	}
	my $ret = AbsenceDB::addGroup($Q::group_id, $Q::name, $Q::pass);
	dbg("ret = [$ret]");

	if ($ret eq 'ok') {
		dbg("looks good");
		reloadTop($q, 'Group Successfully Modified', '_top',
			'Finished. Click on button below to reload top page');
	} else {
		dbg("looks bad");
		abslog("error modifying group_id [$Q::group_id] ret = [$ret]");
		reloadDisplay($q, 'Error modifying group',
			"modify-group [$Q::name] failed.");
	}
	exit;
}

sub modifyGroupForm
{
	my $q = shift;

	my $gref = AbsenceDB::getGroup($Q::group_id);

	if (!defined($gref->{name})) {
		reloadTop($q, "Modify Group $gref->{name}", 'display',
			"Modify Group $gref->{name}", 'display',
			'Group disappeared');
	}

	print $q->header(-expires => '-1d');
	print $q->start_html(
		-title		=> "Absence: Modify Group $gref->{name}",
		-target		=> 'display',
		-BGCOLOR	=> '#AAA0D8',
	);

	print $q->h1('Modify a Group');
	print $q->start_form;
	print $q->hidden(-name => 'group_id', -default => $Q::group_id);
	print "<TABLE>\n";
	print "<TR><TH ALIGN=right>Group Name</TH><TD>\n";
	print $q->textfield(
		-name		=> 'name',
		-size		=> 20,
		-maxlength	=> 20,
		-value		=> $gref->{name},
	);
	print "</TD></TR>\n";
	print "</TD></TR></TABLE>\n";

	print $q->submit(action => 'Submit Group Modification');
	print $q->end_form();
	exit;
}

sub makeGroupCheckboxes
{
	my ($q, $gref) = @_;
	#my @groups = sort {$a <=> $b} AbsenceDB::getGroups();
	my @groups = sort {$a <=> $b} AbsenceAuthorization::getAdminGroups($AUTH_USER);

	my $labref;
	my $name;
	foreach my $group_id (@groups) {
		$name = AbsenceDB::getGroup($group_id, 'name');
		$labref->{$group_id} = $name;
	}

	my @sorted = sort {lc($labref->{$a}) cmp lc($labref->{$b})} @groups;

	print $q->checkbox_group(
		-name		=> 'groups',
		-values		=> \@sorted,
		-default	=> $gref,
		-labels		=> $labref,
		-linebreak	=> 'true',
	);
}

sub makeGroupPopup
{
	my ($q, $default, $form, $action) = @_;

	my @addit;
	if ($form) {
		@addit = (-onChange => "JavaScript:document.${form}.submit('$action');"),
	}

	my @groups = AbsenceAuthorization::getAdminGroups($AUTH_USER);
	my $labref;
	foreach my $group_id (@groups) {
		$labref->{$group_id} = AbsenceDB::getGroup($group_id, 'name');
	}

	my @sorted = sort {lc($labref->{$a}) cmp lc($labref->{$b})} @groups;
	print $q->popup_menu(
		-name   	=> 'group_id',
		-values		=> \@sorted,
		-labels		=> $labref,
		-default	=> $default,
		@addit,
	);
}

sub deletePerson
{
	my $q = shift;

	# get gid first
	my $gref = AbsenceDB::getPerson($Q::person_id, 'group');

	my $ret = AbsenceDB::deletePerson($Q::person_id);
	if ($ret eq 'ok') {
		print $q->redirect($MAIN_SCRIPT);
		foreach my $gid (@{$gref}) {
			removeMonthImages($gid);
		}
	} elsif ($ret eq 'bad-id') {
		reloadDisplay($q, 'Error deleting person',
			'Person was deleted by someone else');
	}
	exit;
}

sub reloadTop
{
	my ($q, $title, $target, $msg) = @_;

	my @args = ($target eq '_top')
		? (-script => "top.location = '$TOP_PAGE';")
		: ();

	print $q->header(-expires => '-1d');
	print $q->start_html(
		-title		=> "Absence: $title",
		-target		=> $target,
		-BGCOLOR	=> '#AAA0D8',
		@args,
	);

	print $q->h1($msg);
	print $q->start_form(
		-action => $TOP_PAGE,
		-method => 'POST',
		-target	=> '_parent',
	);
	print $q->submit(action => 'Reload Calendar');
	print $q->end_form();
	print $q->end_html();
	exit 0;
}

sub reloadDisplay
{
	my ($q, $title, $msg) = @_;

	print $q->header(-expires => '-1d');
	print $q->start_html(
		-title => "Absence: $title",
		-target => 'display',
	);

	print $q->h1($msg);
	print $q->start_form(
		-action => $MAIN_SCRIPT,
		-method => 'POST',
	);
	print $q->submit(action => 'Reload Calendar');
	print $q->end_form();
	print $q->end_html();
	exit 0;
}

sub dbg
{
	my $msg = shift;
	$DEBUG || return;
	abslog("DEBUG: absence-manage: $msg");
}

sub old_dbg
{
	$DEBUG || return;

	my $msg = shift;
	my $fh = FileHandle->new(">>$DEBUG_OUT");
	defined($fh) || die "open $DEBUG_OUT for appending";
	print $fh "$msg\n";
	$fh->close;
}

sub dumpAcl
{
	my ($msg, $acl) = @_;

	dbg("dumpAcl: $msg");
	dbg("  PERSON: ".join(',', @{$acl->{p}}));
	dbg("  GROUPS: ".join(',', @{$acl->{g}}));
}

sub aclForm
{
	my ($q, $form, $uid) = @_;

	my $js_target = 'document.'.$form;

	my $thing = ($form =~/_person$/) ? 'person' : 'user';
	my $verb = ($form =~/add/) ? 'adding' : 'modifying';

	my ($pdref, $gdref);

	$pdref = {};
	$gdref = {};

	my ($perm, $person, $group);

	my %plt = (
		r	=> 'read',
		w	=> 'write',
		a	=> 'admin',
	);

	if ($form ne 'add_user') {
		my (@pdefs, @gdefs);

		if ($uid) {
			my $acl = AbsenceAuthorization::getAdminAcls($AUTH_USER, $uid);
			if ($DEBUG) {
				my $realacl = AbsenceDB::getAcl($uid);
				dumpAcl("aclForm: form=$form, uid=$uid, REAL:", $realacl);
				dumpAcl("aclForm: form=$form, uid=$uid, PRUNED:", $acl);
			}
			@pdefs = @{$acl->{p}};
			@gdefs = @{$acl->{g}};
		} else {
			my $tmp = AbsenceConfig::fetch('pacl_defaults');
			@pdefs = split(/,/, $tmp);
			$tmp = AbsenceConfig::fetch('gacl_defaults');
			@gdefs = split(/,/, $tmp);
		}

		foreach my $rule (@pdefs) {
			($person) = ($rule =~ /^w:(.*)$/);
			if ($person eq 'self') {
				$pdref->{'self:write'} = 'Write:self';
			} elsif ($person =~ /^(\d+)$/) {
				my $pid = $1;
				my $name = AbsenceDB::getPerson($pid, 'name');
				$pdref->{"$pid:write"} = "Write:$name";
			} else {
				die "strange person rule: [$rule]";
			}
		}

		foreach my $rule (@gdefs) {
			($perm, $group) = ($rule =~ /^(a|w|r):(.*)$/);
			if ($group =~ /^(all|self)$/) {
				$gdref->{"$group:$plt{$perm}"} = ucfirst($plt{$perm}).":$group";
			} elsif ($group =~ /^(\d+)$/) {
				my $gid = $1;
				my $name = AbsenceDB::getGroup($gid, 'name');
				$gdref->{"$gid:$plt{$perm}"} = ucfirst($plt{$perm}).":$name";
			} else {
				die "strange group rule: [$rule]";
			}
		}
	}

	print qq[<TR><TD></TD><TD COLSPAN=99>\n];
	javaScriptWarning();
	print qq[</TH></TR>\n];

	######################### GROUP START #########################
	#------------------------------------------------------------
	# GROUP stuff
	#------------------------------------------------------------
	print qq[<TR><TH ALIGN=right VALIGN=top>Group ACL<BR>Management</TH>
		<TD COLSPAN=2>\n];

	# TABLE wrapper
	print qq[<TABLE BORDER=1><TR><TD>\n];

	print qq[<TABLE>\n];

	#-------------------------------------------
	# instructions
	#-------------------------------------------
	if ($thing eq 'person') {
		print qq[<TR><TH COLSPAN=99 ALIGN=left><SMALL>
			Use Group ACLs to control what
			the person you are $verb can do with her own group or other
			groups.  Choose a permission (read/write/admin), choose a
			target group for that permission to act upon, then click on
			"Add" to add the 'rule' to the list on the right.  The group
			"self" refers to all groups to which the person belongs.<BR>
			Permissions: read: view group, write: add/change/delete
			absences, admin: add/change/delete people.
			<BR><HR>
			</SMALL></TH></TR>\n];
	} else {
		print qq[<TR><TH COLSPAN=99 ALIGN=left><SMALL>
			Use Group ACLs to control what
			the user you are $verb can do with
			groups.  Choose a permission (read/write/admin), choose a
			target group for that permission to act upon, then click on
			"Add" to add the 'rule' to the list on the right.
			<BR>
			Permissions: read: view group, write: add/change/delete
			absences, admin: add/change/delete people.
			<BR><HR>
			</SMALL></TH></TR>\n];
	}

	print qq[<TR>
		<TD VALIGN=top WIDTH="100">
		Permission:<BR>\n];

	my @perms = qw(read write admin);
	$lref = {};
	foreach my $p (@perms) {
		$lref->{$p} = ucfirst($p);
	}

	print $q->radio_group(
			-name		=> 'groupverb',
			-values		=> \@perms,
			-linebreak	=> 'true',
			-labels		=> $lref,
		),
		qq[</TD>
		<TD VALIGN=top WIDTH="150">
		Group:<BR>\n];

	$lref = undef;
	my @groups = AbsenceAuthorization::getAdminGroups($AUTH_USER);
	foreach my $gid (@groups) {
		$lref->{$gid} = AbsenceDB::getGroup($gid, 'name');
	}
	$lref->{self} = 'self';
	$lref->{all} = 'all';
	@list = ();
	if (($form eq 'add_person') ||
		(($form eq 'mod_person') &&
		 (AbsenceAuthorization::selfAllowed($AUTH_USER, $uid))))
	{
		@list = ('self');
	}
	AbsenceAuthorization::adminAll($AUTH_USER) && push(@list, 'all');
	push(@list, sort {lc($lref->{$a}) cmp lc($lref->{$b})} @groups);
	print $q->popup_menu(
		-name		=> 'grouplist',
		-values		=> \@list,
		-labels		=> $lref,
		-size		=> 5,
		-multiple	=> undef,
	);
	print qq[</TD>
		<TD WIDTH="100">\n],
		$q->button(
			-name	=> 'add_group',
			-value	=> 'Add',
			-onClick	=> "add('group', $js_target)",
		), "<BR>\n",
		$q->button(
			-name	=> 'del_group',
			-value	=> 'Delete',
			-onClick	=> "del('group', $js_target)",
		),
		qq[</TD>
		<TD WIDTH="150">\nSelected Rules:<BR>\n],
		$q->popup_menu(
			-name		=> 'group_acl',
			-values		=> [keys(%{$gdref})],
			-labels		=> $gdref,
			-size		=> 5,
			-multiple	=> undef,
		),
		qq[</TD>\n</TR>\n];

	# HR
	#print qq[<TR><TD COLSPAN=4><HR></TD></TR>\n];

	print qq[</TABLE>\n];

	# TABLE wrapper
	print qq[</TD></TR></TABLE>\n];
	######################### GROUP END #########################

	print qq[</TD></TR>\n];

	######################### PERSON START #########################
	#------------------------------------------------------------
	# PERSON stuff
	#------------------------------------------------------------
	print qq[<TR><TH ALIGN=right VALIGN=top>Person ACL<BR>Management</TH>
		<TD COLSPAN=2>\n];

	# TABLE wrapper
	print qq[<TABLE BORDER=1><TR><TD>\n];

	print qq[<TABLE>\n];

	#-------------------------------------------
	# instructions
	#-------------------------------------------
	if ($thing eq 'person') {
		print qq[<TR><TH COLSPAN=99 ALIGN=left><SMALL>
			Use Person ACLs to control what
			the person you are $verb can do with herself or other
			people in groups which are visible to her (see above).
			The only permission available is "write". Choose a
			target person for whom "write" access is to be allowed,
			then click "Add" to add the 'rule' to the list on the right.
			The special person "self" refers to the person you are $verb.
			<BR>
			"write" access means adding, modifying, and deleting absences
			is allowed.
			<BR><HR>
			</SMALL></TH></TR>\n];
	} else {
		print qq[<TR><TH COLSPAN=99 ALIGN=left><SMALL>
			Use Person ACLs to control what
			the user you are $verb can do with
			people in groups which are visible to her (see above).
			The only permission available is "write". Choose a
			target person for whom "write" access is to be allowed,
			then click "Add" to add the 'rule' to the list on the right.
			The special person "self" refers to the person you are $verb.
			<BR>
			"write" access means adding, modifying, and deleting absences
			is allowed.
			<BR><HR>
			</SMALL></TH></TR>\n];
	}

	print qq[<TR>
		<TD VALIGN=top WIDTH="100">
		Permission:<BR>
		<INPUT TYPE=radio NAME="personverb" VALUE="write" CHECKED>Write
		</TD>
		<TD VALIGN=top WIDTH="150">
		Person:<BR>\n];

	my $lref;
	my @people = AbsenceAuthorization::getAdminPeople($AUTH_USER);
	foreach my $pid (@people) {
		$lref->{$pid} = AbsenceDB::getPerson($pid, 'name');
	}
	$lref->{self} = 'self';
	my @list = ($form =~ /_person$/) ?  ('self') : ();
	push(@list, sort {lc($lref->{$a}) cmp lc($lref->{$b})} @people);

	print $q->popup_menu(
		-name		=> 'personlist',
		-values		=> \@list,
		-labels		=> $lref,
		-size		=> 5,
		-multiple	=> undef,
	);
		
	print qq[</TD>\n
		<TD WIDTH="100">\n],
		$q->button(
			-name	=> 'add_person',
			-value	=> 'Add',
			-onClick	=> "add('person', $js_target)",
		),
		"<BR>\n",
		$q->button(
			-name	=> 'del_person',
			-value	=> 'Delete',
			-onClick	=> "del('person', $js_target)",
		),
		qq[</TD>
		<TD WIDTH="150">\nSelected Rules:<BR>],
		$q->popup_menu(
			-name		=> 'person_acl',
			-values		=> [keys(%{$pdref})],
			-labels		=> $pdref,
			-size		=> 5,
			-multiple	=> undef,
		),
		qq[</TD></TR>\n];
		#print qq[<TR><TD COLSPAN=4 ALIGN="center">\n];
		#print qq[<SMALL>NOTE: you cannot add "write" permission for a
		#	person in a group for which the user does not have
		#	"read" permission.</SMALL>\n];
		#print qq[</TD></TR>\n];

		print qq[<TR>\n</TABLE>\n];

	# TABLE wrapper
	print qq[</TD></TR></TABLE>\n];
	######################### PERSON END #########################

	print qq[</TD></TR>\n];
}

sub includeJsAclScript
{
	my $path = AbsenceConfig::fetch('js_acl_script');

	#print qq[<SCRIPT LANGUAGE="JavaScript" SRC="$path"></SCRIPT>\n];
	my $fh = FileHandle->new($path) || die "open [$path] to read";
	print qq[<SCRIPT LANGUAGE="JavaScript">\n<!--\n];
	while(<$fh>) {
		print $_;
	}
	$fh->close;
	print "-->\n</SCRIPT>\n";
}

sub javaScriptWarning
{
	print <<_EOF_;
<NOSCRIPT>
<FONT SIZE=4 COLOR="red">
You will not be able to manage ACLs because your browser<BR>
does not support JavaScript or you have not enabled it.
</FONT>
</NOSCRIPT>
_EOF_
}

__END__

=head1 NAME

absence-manage.pl - manage people and groups

=head1 DESCRIPTION

C<absence-manage.pl> is responsible for displaying some forms for
the management of people and groups, and handling the processing
after the user clicks on submit in one of the forms.

=cut
